package qsrtdemo;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import qsrtdemo.control.ControlPanel;
import qsrtdemo.model.Quicksort;
import qsrtdemo.view.DescriptionPanel;
import qsrtdemo.view.PartitionsPanel;
import qsrtdemo.view.StepPanel;

/**
 * The main class. This sets up and links together all the separate parts of the
 * program, and displays them in a window.
 *
 * @author Michael Hawes
 */
public class Main extends JFrame {
    
    private Quicksort model;
    private ControlPanel control;
    private DescriptionPanel dp;
    private StepPanel sp;
    private PartitionsPanel pp;
    
    public Main() {
        Container pane = getContentPane();
        
        model = new Quicksort();
        control = new ControlPanel();
        dp = new DescriptionPanel();
        sp = new StepPanel();
        sp.setBorder(BorderFactory.createLineBorder(Color.gray));
        pp = new PartitionsPanel();
        control.addObserver(model);
        model.addObserver(dp);
        model.addObserver(sp);
        model.addObserver(pp);
        pp.setListener(model);
        model.setEndListener(control);
        sp.setAnimListener(model);
        
        JPanel viewPanel = new JPanel(new BorderLayout());
        viewPanel.add(sp, BorderLayout.NORTH);
        viewPanel.add(pp, BorderLayout.CENTER);
        viewPanel.add(dp, BorderLayout.SOUTH);
        
        pane.add(viewPanel, BorderLayout.CENTER);
        pane.add(control,BorderLayout.SOUTH);

        setTitle("QuickSort Demonstration (Copyright Michael Hawes 2009)");
        setSize(800,600);
    }

    public static void main(String[] args) {
        // Need to set up interface in Event Dispatch Thread
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                Main frame = new Main();
                frame.setVisible(true);
                frame.setDefaultCloseOperation(EXIT_ON_CLOSE);
            }
        });
    }
    
}
