package qsrtdemo.model;

import java.util.Observable;

/**
 * Example class to demonstrate Quicksort class
 * 
 * @author Michael Hawes
 */
public class ModelTest {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        Quicksort sort = new Quicksort();
        sort.update(new Observable(), new InitSort(new RandomiseVars(10, 100)));
        
        System.out.println("Quicksort:\n");
        System.out.println(sort);
        getData(sort);
    }
    
    // example method to find data about steps
    private static void getData(Quicksort sort) {
        System.out.println("Looking at steps in the base partition:");
        Partition basePartition = sort.getBasePartition();
        PartitionStep step;
        for(int i=0; i<basePartition.getNumSteps(); i++) {
            step = basePartition.getStep(i);
            if(step.getClass().equals(SwapStep.class)) {
                System.out.println("Found a swap step");
                SwapStep stepA = (SwapStep)step;
                int[] array = stepA.getArray();
                int indexA = stepA.getIndexA();
                int indexB = stepA.getIndexB();
                System.out.println("Values to swap are "+array[indexA]+" and "+array[indexB]+" at positions "+indexA+" and "+indexB);
                int swapType = stepA.getSwapType();
                switch(swapType) {
                    case SwapStep.SWAP_MAKEPIVOTCORRECT:
                        System.out.println("Step is moving the pivot to its correct position");
                        break;
                    case SwapStep.SWAP_MAKEPIVOTSAFE:
                        System.out.println("Step is moving the pivot to safety at the end of the current partition");
                        break;
                    case SwapStep.SWAP_PIVOTSTART:
                        System.out.println("Step is moving the pivot to the start of the current partition, having discovered that it is the maximum value in the partition");
                        break;
                    case SwapStep.SWAP_SWAPVALUES:
                        System.out.println("Step is swapping two values during the process of ensuring all values before than the pivot are greater than it and all values after it are smaller");
                        break;
                    default:
                        System.out.println("Invalid step type");
                }
            } else if(step.getClass().equals(ValStep.class)) {
                System.out.println("Found a step involving a single value");
                ValStep stepA = (ValStep)step;
                int[] array = stepA.getArray();
                int valIndex = stepA.getValIndex();
                System.out.println("Value is "+array[valIndex]+" at position "+valIndex);
                int valType = stepA.getValType();
                switch(valType) {
                    case ValStep.VAL_PIVOTEND:
                        System.out.println("Step is seeing that the pivot is the minimum value");
                        break;
                    case ValStep.VAL_SINGLEVAL:
                        System.out.println("Step is seeing that quicksort has been called on a single value");
                        break;
                    default:
                        System.out.println("Invalid step type");
                }
            }
            System.out.println();
        }
    }
    
    // mike's testing method
    public static String createIndent(int indentLvl) {
        String indent="";
        for(int i=0; i<indentLvl; i++) {
            indent = indent + "  ";
        }
        return indent;
    }
}
