package qsrtdemo.model;

/**
 * This class stores all information about a given partition in the process of
 * sorting an array. It stores the steps taken to sort the partition, any new
 * partitions created in the process of sorting this one, and other information.
 *
 * @author Michael Hawes
 */
public class Partition {
    
    public static final int maxSteps = 100;
    
    private int[] array;
    private int leftIndex;
    private int rightIndex;
    private PartitionStep[] steps;
    private int numSteps;
    private Partition leftPart;
    private Partition rightPart;
    private int finalisedIndex=-1;
    private boolean editable=true;
    
    public Partition(int[] currArray, int leftIndex, int rightIndex) {
        array = new int[currArray.length];
        System.arraycopy(currArray, 0, array, 0, currArray.length);
        this.leftIndex = leftIndex;
        this.rightIndex = rightIndex;
        steps = new PartitionStep[maxSteps];
        numSteps = 0;
    }
    
    public void addStep(PartitionStep step) {
        if(editable) {
            steps[numSteps] = step;
            numSteps++;
        }
    }
    
    public void setLeftPart(Partition partition) {
        if(editable) leftPart = partition;
    }
    
    public void setRightPart(Partition partititon) {
        if(editable) rightPart = partititon;
    }
    
    public int getLeftIndex() {
        return leftIndex;
    }
    
    public int getRightIndex() {
        return rightIndex;
    }
    
    public int[] getArray() {
        return array;
    }
    
    public int getNumSteps() {
        return numSteps;
    }
    public PartitionStep getStep(int index) {
        return steps[index];
    }
    
    public Partition getLeftPartition() {
        return leftPart;
    }
    public Partition getRightPartition() {
        return rightPart;
    }
    
    public int getFinalisedIndex() {
        return finalisedIndex;
    }
    
    public void setFinalisedIndex(int finalised) {
        if(editable) finalisedIndex = finalised;
    }

    public int getPivot() {
        return array[steps[0].getPivot()];
    }
    
    public void finalise() {
        editable=false;
    }
    
    // mike's testing method
    @Override
    public String toString() {
        return toString(0);
    }
    
    // mike's testing method
    public String toString(int indentLvl) {
        String indent = ModelTest.createIndent(indentLvl);
        String output = indent+"Current array:\n"+indent+arrayString(array)+"\n";
        output = output + indent+"Partitioning from index "+leftIndex+" to index "+rightIndex+":\n\n";
        for(int i=0; i<numSteps; i++) {
            output = output + steps[i].toString(indentLvl)+"\n"+indent+arrayString(steps[i].getArray())+"\n";
        }
        output = output + "\n";
        
        if(leftPart!=null) output = output + indent+"Left partition:\n" + leftPart.toString(indentLvl+1);
        if(rightPart!=null) output = output + indent+"Right partition:\n" + rightPart.toString(indentLvl+1);
        
        return output;
    }
    
    // mike's testing method
    private String arrayString(int[] array) {
        String arrayStr = "";
        for(int i=0; i<array.length; i++) {
            arrayStr = arrayStr + array[i] + " ";
        }
        return arrayStr;
    }
}
