package qsrtdemo.model;

/**
 * This is the step type for swapping two values in the array
 *
 * @author Michael Hawes
 */
public class SwapStep extends PartitionStep {
    
    public static final int SWAP_MAKEPIVOTSAFE = 0; // the new position of the pivot will always be index B
    public static final int SWAP_SWAPVALUES = 1;
    // below here are step types that mean a new partition is to be entered
    public static final int SWAP_MAKEPIVOTCORRECT = 2; // the new position of the pivot will always be index A
    public static final int SWAP_PIVOTSTART = 3; // the new position of the pivot will always be index A
    
    private int swapType;
    private int indexA;
    private int indexB;
    
    public SwapStep(int[] newArray, int newPivot, int swapType, int indexA, int indexB) {
        super(newArray, newPivot);
        this.swapType = swapType;
        this.indexA = indexA;
        this.indexB = indexB;
    }
    
    public int getSwapType() {
        return swapType;
    }
    
    public int getIndexA() {
        return indexA;
    }
    
    public int getIndexB() {
        return indexB;
    }
    
    // mike's testing method
    @Override
    public String toString(int indentLvl) {
        String indent = ModelTest.createIndent(indentLvl);
        String output = indent;
        switch(swapType) {
            case SWAP_MAKEPIVOTSAFE:
                output = output + "Moving pivot at position "+indexA+" to end of partition";
                break;
            case SWAP_SWAPVALUES:
                output = output + "Swapping value at position "+indexA+" with value at position "+indexB;
                break;
            case SWAP_MAKEPIVOTCORRECT:
                output = output + "Moving pivot to its correct position at position "+indexA+", will now partition to the left and right";
                break;
            case SWAP_PIVOTSTART:
                output = output + "Pivot is the maximum in the partition, moving it to beginning";
                break;
            default:
                output = output + "Invalid step type";
        }
        return output;
    }
    
}
