package qsrtdemo.view;

import java.awt.BasicStroke;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;

/**
 *
 * @author Michael Hawes
 */
public class ArrayNode {
    
    public static final float gapX = 20;
    public static final float strPadding = 5;
    public static final float defaultSize = 50;
    
    private static float width = defaultSize;
    private static float height = defaultSize;
    
    private float x;
    private float y;
    private float nextX;
    private float nextY;
    private String str;
    private float strOffsetX;
    private float strOffsetY;
    private ArrayNodeAnim anim;
    
    public static void setWidth(float width) {
        ArrayNode.width = width;
    }
    public static void setHeight(float height) {
        ArrayNode.height = height;
    }
    public static float getWidth() {
        return width;
    }
    public static float getHeight() {
        return height;
    }
    
    /**
     * Creates a new instance of ArrayNode
     * @param val
     * @param startX
     * @param startY
     * @param font
     * @param context
     * @param stroke
     */
    public ArrayNode(int val, float startX, float startY, Font font, FontRenderContext context, BasicStroke stroke) {
        initNode(val, startX, startY, font, context, stroke);
    }
    
    /**
     * Creates a new instance of ArrayNode
     * @param val 
     * @param prevNode
     * @param font
     * @param context
     * @param stroke 
     */
    public ArrayNode(int val, ArrayNode prevNode, Font font, FontRenderContext context, BasicStroke stroke) {
        float x = prevNode.getNextX();
        float y = prevNode.getNextY();
        initNode(val, x, y, font, context, stroke);
    }
    
    public void setAnim(ArrayNodeAnim anim) {
        this.anim = anim;
    }
    
    private void initNode(int val, float x, float y, Font font, FontRenderContext context, BasicStroke stroke) {
        
        str = ""+val;
        this.x = x;
        this.y = y;
        nextY = y;
        
        GlyphVector glyphs = font.createGlyphVector(context, str);
        Rectangle2D bounds = glyphs.getVisualBounds();
        
        float strHeight = (float)bounds.getHeight();
        float strWidth = (float)bounds.getWidth();
        float strokePad = (float)(stroke.getLineWidth()/2.0);
        float fullPadding = strPadding*2+stroke.getLineWidth();
        
        float spaceX = width-fullPadding-strWidth;
        strOffsetX = strokePad + strPadding + (float)(spaceX/2.0);
        float spaceY = height-fullPadding-strHeight;
        strOffsetY = strokePad + strPadding + strHeight + (float)(spaceY/2.0);
        
        nextX = x + width + gapX;
    }
    
    public static Dimension getDimensions(String str, Font font, FontRenderContext context, BasicStroke stroke) {
        
        GlyphVector glyphs = font.createGlyphVector(context, str);
        Rectangle2D bounds = glyphs.getVisualBounds();
        
        float strHeight = (float)bounds.getHeight();
        float strWidth = (float)bounds.getWidth();
        float strokePad = (float)(stroke.getLineWidth()/2.0);
        float fullPadding = strPadding*2+stroke.getLineWidth();
        float defaultSpace = defaultSize-fullPadding;
        float width;
        float height;
        if(strWidth<defaultSpace) {
            width = 50;
        } else {
            width = strWidth + fullPadding;
        }
        if(strHeight<(50-fullPadding)) {
            height = 50;
        } else {
            height = strHeight + fullPadding;
        }
        
        Dimension dim = new Dimension();
        dim.setSize(width, height);
        return dim;
    }
    
    public float getNextX() {
        return nextX;
    }
    
    public float getNextY() {
        return nextY;
    }
    
    public float getTopLeftX() {
        return x;
    }
    public float getTopLeftY() {
        return y;
    }
    public float getBottomRightX() {
        return x+width;
    }
    public float getBottomRightY() {
        return y+height;
    }
    
    public void moveX(float xMove) {
        x += xMove;
        nextX += xMove;
    }
    
    public float getX() {
        return x;
    }
    public float getY() {
        return y;
    }
    public float getStrX() {
        return x+strOffsetX;
    }
    public float getStrOffsetX() {
        return strOffsetX;
    }
    public float getStrY() {
        return y+strOffsetY;
    }
    public float getStrOffsetY() {
        return strOffsetY;
    }
    public String getStr() {
        return str;
    }
    
    public void drawNode(Graphics2D g) {
        if(anim!=null) anim.drawNode(g, this);
    }
    
}
