package qsrtdemo.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JPanel;
import qsrtdemo.model.Partition;
import qsrtdemo.model.Quicksort;

/**
 *
 * @author Michael Hawes
 */
public class PartitionsPanel extends JPanel implements MouseListener, Observer {
    
    private static final Font font = new Font("SansSerif", Font.BOLD, 12);
    private static final Font titleFont = new Font("SansSerif", Font.BOLD, 22);
    private static final BasicStroke stroke = new BasicStroke(2);
    private static final RenderingHints rh = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    private static final String title = "Tree of partitions";
    private static final int padX=5;
    private static final int padY=5;
    private static final int titlePad = 60;
    private static final int titleY = 22+padY;
    
    private PartitionNode base;
    private ActionListener listener;
    
    /** Creates a new instance of RecursionPanel */
    public PartitionsPanel(){
        super.addMouseListener(this);
        setPreferredSize(new Dimension(0, padY*2+titlePad));
    }
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHints(rh);
        g2.setStroke(stroke);
        if(base!=null) {
            g2.setFont(font);
            
            base.draw(g2);
        }
        g2.setColor(Color.black);
        g2.setFont(titleFont);
        g2.drawString(title, padX, titleY);
    }
    
    public void setListener(ActionListener l) {
        listener = l;
    }
    
    public void mousePressed (MouseEvent e) {
        if(listener!=null && base!=null) {
            int x = e.getX();
            int y = e.getY();
            int partNo=base.getPartNumAt(x,y);
            if(partNo>-1) {
                listener.actionPerformed(new ActionEvent(this, 0, "part"+partNo));
            }
        }
    }
    
    public void mouseReleased (MouseEvent e) {}
    
    public void mouseEntered (MouseEvent e) {}
    
    public void mouseExited (MouseEvent e) {}
    
    public void mouseClicked (MouseEvent e) {}
    
    public void update(Observable o, Object arg) {
        if(arg.equals(Quicksort.UPDATE_PARTITION_NEXT) || arg.equals(Quicksort.UPDATE_PARTITION_PREV)){
            Quicksort sort = (Quicksort)o;
            PartitionNode.setPartSelected(sort.getCurrPartIndex());
            repaint();
        }else if(arg.equals(Quicksort.UPDATE_BEGIN)){
            Quicksort sort = (Quicksort)o;
            setBasePartition(sort.getBasePartition());
            repaint();
        }
    }
    
    private void setBasePartition(Partition basePart) {
        Graphics2D g = (Graphics2D)getGraphics();
        PartitionNode.init(basePart, font, g.getFontRenderContext(), titlePad);
        base = PartitionNode.getBase();
        Dimension size = base.getBounds();
        setPreferredSize(size);
        setMaximumSize(size);
        refreshAlignment();
    }
    
    public void doLayout() {
        super.doLayout();
        
        refreshAlignment();
    }
    
    private void refreshAlignment() {
        if(base!=null) {
            Dimension dim = getSize();
            float width = (float)dim.getWidth();
            Dimension treeBounds = base.getBounds();
            int newXStart = (int)((width-treeBounds.getWidth())/2.0);
            base.setXStart(newXStart);
        }
    }
    
}
