package qsrtdemo.view;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;

/**
 *
 * @author Michael Hawes
 */
public class SwapNodeAnim implements ArrayNodeAnim {
    
    private Color col;
    private ArrayNode from;
    private long startTime;
    private double speed;
    private double distY;
    private ActionListener endListener;
    private boolean finished=false;
    
    private static final double defaultY=70;
    
    /**
     * 
     * @param colour
     * @param swapWith
     * @param speed The speed at which the node should move (pixels/s)
     */
    public SwapNodeAnim(Color colour, ArrayNode origNode, int speed, ActionListener endListener) {
        col = colour;
        from = origNode;
        startTime = System.currentTimeMillis();
        this.speed = speed/1000.0;
        distY = defaultY;
        this.endListener = endListener;
    }
    
    public void drawNode(Graphics2D g, ArrayNode node) {
        double alongPath = speed*(System.currentTimeMillis()-startTime);
        double distX = from.getX()-node.getX();
        if(alongPath>distY) {
            double midPathDist = distY+Math.abs(distX);
            if(alongPath>midPathDist) {
                double pathDist = midPathDist+distY;
                if(alongPath>=pathDist) {
                    float x = node.getX();
                    float y = node.getY();
                    drawNode(g, x, y, node.getWidth(), node.getHeight(), node.getStr(), x+node.getStrOffsetX(), y+node.getStrOffsetY());
                    if(!finished) {
                        finished = true;
                        ActionEvent e = new ActionEvent(this, -1, "animFinished");
                        endListener.actionPerformed(e);
                    }
                } else {
                    float x = node.getX();
                    float y = node.getY() + (float)(pathDist-alongPath);
                    drawNode(g, x, y, node.getWidth(), node.getHeight(), node.getStr(), x+node.getStrOffsetX(), y+node.getStrOffsetY());
                }
            } else {
                float x;
                if(distX>0) {
                    x = from.getX()-(float)(alongPath-distY);
                } else {
                    x = from.getX()+(float)(alongPath-distY);
                }
                float y = from.getY()+(float)distY;
                drawNode(g, x, y, node.getWidth(), node.getHeight(), node.getStr(), x+node.getStrOffsetX(), y+node.getStrOffsetY());
            }
        } else {
            float x = from.getX();
            float y = from.getY()+(float)alongPath;
            drawNode(g, x, y, node.getWidth(), node.getHeight(), node.getStr(), x+node.getStrOffsetX(), y+node.getStrOffsetY());
        }
    }
    
    private void drawNode(Graphics2D g, float x, float y, float width, float height, String str, float strX, float strY) {
        Rectangle2D shape = new Rectangle2D.Float(x, y, width, height);
        g.setColor(Color.WHITE);
        g.fill(shape);
        g.setColor(col);
        g.draw(shape);
        
        g.setColor(Color.BLACK);
        g.drawString(str, strX, strY);
    }
    
}
